#include "stdafx.h"
#include "RecentContactsPersistencePolicy.h"


CSqlitePolicy::CSqlitePolicy(int nRecordSize):m_nRecordSize(nRecordSize)
{
	m_bIsOpened = FALSE;
	m_bIsOpened = Open();
}

CSqlitePolicy::~CSqlitePolicy()
{
	if(m_bIsOpened)
	{
		m_bIsOpened = !Close();
	}
}

BOOL CSqlitePolicy::Add(CString strMyUserId, CString strContactId,
						const CTime& timeContact)
{
	if(!m_bIsOpened)
	{
		return FALSE;
	}

	string stringUserId = UnicodeToUTF8(strMyUserId);
	string stringContactId = UnicodeToUTF8(strContactId);
	INT64 nTime = CTimeToInt(timeContact);
	sqlite3_stmt* pVM;
	char sql[1024];
	ZeroMemory(sql, sizeof(sql));

	/*sprintf_s(sql, sizeof(sql), 
		"if exists
		( select userid from my_recent_contacts\
		where userid = ? AND contactid = ?)\
		update my_recent_contacts set contacttime = ? where \
		userid = ? AND contactid = ?\
		else\
		INSERT INTO my_recent_contacts (userid,\
		contactid, contacttime) VALUES (?, ?, ?)");	*/


	sprintf_s(sql, sizeof(sql), 
		"INSERT INTO my_recent_contacts (userid,\
			contactid, contacttime) VALUES (?, ?, ?)");

	sqlite3_prepare(m_pSQLite, sql, -1, &pVM, NULL);
	sqlite3_reset(pVM);
	sqlite3_bind_text(pVM, 1, stringUserId.c_str(), stringUserId.length(), SQLITE_STATIC);
	sqlite3_bind_text(pVM, 2, stringContactId.c_str(), stringContactId.length(), SQLITE_STATIC);
	sqlite3_bind_int64(pVM, 3, nTime);
	/*sqlite3_bind_text(pVM, 4, strUserID.c_str(), strUserID.length(), SQLITE_STATIC);
	sqlite3_bind_text(pVM, 5, strContactId, strContactId.length(), SQLITE_STATIC);
	sqlite3_bind_text(pVM, 6, strUserID.c_str(), strUserID.length(), SQLITE_STATIC);
	sqlite3_bind_text(pVM, 7, strContactId, strContactId.length(), SQLITE_STATIC);
	sqlite3_bind_int64(pVM, 8, nTime);   */

	int nRet = sqlite3_step(pVM);

	return (BOOL)(SQLITE_DONE == nRet);

}

BOOL CSqlitePolicy::Delete( CString strMyUserId, CString strContactId )
{

	if(!m_bIsOpened)
	{
		return FALSE;
	}

	string stringUserID = UnicodeToUTF8(strMyUserId);
	string stringContactId = UnicodeToUTF8(strContactId);

	sqlite3_stmt* pVM;
	char sql[1024];
	ZeroMemory(sql, sizeof(sql));

	sprintf_s(sql, sizeof(sql), "delete from my_recent_contacts where userid = ?\
								and contactid = ?");
	sqlite3_prepare(m_pSQLite, sql, -1, &pVM, NULL);
	sqlite3_reset(pVM);

	sqlite3_bind_text(pVM, 1, stringUserID.c_str(), stringUserID.length(), SQLITE_STATIC);
	sqlite3_bind_text(pVM, 2, stringContactId.c_str(), stringContactId.length(), SQLITE_STATIC);
	int nRet = sqlite3_step(pVM);

	return (BOOL)(SQLITE_DONE == nRet);
}

BOOL CSqlitePolicy::GetAll( CString strMyUserId, deque<CString>* pDeqContactId )
{
	if(!m_bIsOpened || NULL == pDeqContactId)
	{
		return FALSE;
	}

	string stringUserID = UnicodeToUTF8( strMyUserId );

	sqlite3_stmt* pVM;
	char sql[1024];
	ZeroMemory(sql, sizeof(sql));

	sprintf_s(sql, sizeof(sql), "SELECT contactid FROM my_recent_contacts WHERE userid = ? ORDER BY contacttime DESC");

	sqlite3_prepare(m_pSQLite, sql, -1, &pVM, NULL);
	sqlite3_reset(pVM);
	sqlite3_bind_text(pVM, 1, stringUserID.c_str(), stringUserID.length(), SQLITE_STATIC);
	int nRet = sqlite3_step(pVM);

	if(nRet != SQLITE_DONE)//there's some record(s)  got
	{
		while(TRUE)
		{
			const char* pResult = (char*)sqlite3_column_text(pVM, 0);
			if(NULL == pResult)
			{
				return FALSE;
			}
			//string stringContactId(pResult);
			//if(m_nRecordSize > pDeqContactId->size())
			//{
				pDeqContactId->push_back(CString(pResult));
			//}
	
			nRet = sqlite3_step(pVM);
			if(nRet != SQLITE_ROW)
			{
				break;
			}
		}


	}

	return TRUE;

}

BOOL CSqlitePolicy::Open()
{
	BOOL bRet = FALSE;
	CString strSQLiteFilePath = GetAppDataNtalkDir() + _T("\\Ntalkbiz.db");
	sqlite3_config(SQLITE_CONFIG_SERIALIZED);
	int nRet = sqlite3_open( UnicodeToUTF8(strSQLiteFilePath).c_str(), &m_pSQLite);
	if(SQLITE_OK == nRet)
	{
		//if not exits table , create one
		//remark : do not use "IF EXISTS" in sqlite
		do
		{
			sqlite3_stmt* pVM;

			char* sql1 = "select count(*) from sqlite_master where type='table' and name='my_recent_contacts'";
			char* sql2 = "create table my_recent_contacts(userid VARCHAR(24) \
				NOT NULL,contactid VARCHAR(24) NOT NULL,contacttime \
				UNSIGNED BIG INT NOT NULL)";
			nRet = sqlite3_prepare(m_pSQLite, sql1, -1, &pVM, NULL);
			if(nRet != SQLITE_OK)
			{
				break;
			}

			nRet = sqlite3_step(pVM);
			if(nRet != SQLITE_ROW)
			{
				break;
			}

			INT64  nCount = sqlite3_column_int64(pVM, 0);

			nRet = sqlite3_step(pVM);
			if(nRet != SQLITE_DONE)
			{
				break;
			}

			//not Exists table
			if(nCount == 0)
			{
				nRet = sqlite3_prepare(m_pSQLite, sql2, -1, &pVM, NULL);
				if(nRet != SQLITE_OK)
				{
					break;
				}

				nRet = sqlite3_step(pVM);
				if(nRet != SQLITE_DONE)
				{
					break;
				}
			}

			bRet = TRUE;
		}
		while(false);
	}
	else
	{
		m_pSQLite = NULL;
	}

	return bRet;
}

BOOL CSqlitePolicy::Close()
{
	if(m_pSQLite != NULL)
	{
		if(SQLITE_OK == sqlite3_close(m_pSQLite))
		{
			return TRUE;
		}
	}

	return FALSE;
}





